@@ -15,11 +15,12 @@
 
 """Manage backends in the current zone."""
 
+from __future__ import annotations
+
 from collections import abc
 import random
 import typing
-from typing import (Any, Dict, Iterable, List,  # noqa: H301
-                    Optional, Type, Union)
+from typing import (Any, Iterable, Optional, Type, Union)  # noqa: H301
 
 from oslo_config import cfg
 from oslo_log import log as logging
@@ -72,7 +73,7 @@ LOG = logging.getLogger(__name__)
 
 class ReadOnlyDict(abc.Mapping):
     """A read-only dict."""
-    def __init__(self, source: Union[dict, 'ReadOnlyDict'] = None):
+    def __init__(self, source: Optional[Union[dict, 'ReadOnlyDict']] = None):
         self.data: dict
         if source is not None:
             self.data = dict(source)
@@ -163,7 +164,7 @@ class BackendState(object):
         self.service = ReadOnlyDict(service)
 
     def update_from_volume_capability(self,
-                                      capability: Dict[str, Any],
+                                      capability: dict[str, Any],
                                       service=None) -> None:
         """Update information about a host from its volume_node info.
 
@@ -289,13 +290,18 @@ class BackendState(object):
                                                 'host': self.host})
             del self.pools[pool]
 
-    def _append_backend_info(self, pool_cap: Dict[str, Any]) -> None:
+    def _append_backend_info(self, pool_cap: dict[str, Any]) -> None:
         # Fill backend level info to pool if needed.
         if not pool_cap.get('volume_backend_name', None):
             pool_cap['volume_backend_name'] = self.volume_backend_name
 
-        if not pool_cap.get('storage_protocol', None):
-            pool_cap['storage_protocol'] = self.storage_protocol
+        protocol = pool_cap.get('storage_protocol', None)
+        if protocol:
+            # Protocols that have variants are replaced with ALL the variants
+            storage_protocol = self.get_storage_protocol_variants(protocol)
+        else:  # Backend protocol has already been transformed with variants
+            storage_protocol = self.storage_protocol
+        pool_cap['storage_protocol'] = storage_protocol
 
         if not pool_cap.get('vendor_name', None):
             pool_cap['vendor_name'] = self.vendor_name
@@ -320,7 +326,12 @@ class BackendState(object):
         self.volume_backend_name = capability.get('volume_backend_name', None)
         self.vendor_name = capability.get('vendor_name', None)
         self.driver_version = capability.get('driver_version', None)
-        self.storage_protocol = capability.get('storage_protocol', None)
+
+        # Protocols that have variants are replaced with ALL the variants
+        protocol = capability.get('storage_protocol', None)
+        self.storage_protocol = self.get_storage_protocol_variants(protocol)
+        if 'storage_protocol' in capability:
+            capability['storage_protocol'] = self.storage_protocol
         self.updated = capability['timestamp']
 
     def consume_from_volume(self,
@@ -370,6 +381,18 @@ class BackendState(object):
                  'thick': self.thick_provisioning_support,
                  'pools': self.pools, 'updated': self.updated})
 
+    @staticmethod
+    def get_storage_protocol_variants(storage_protocol):
+        if storage_protocol in constants.ISCSI_VARIANTS:
+            return constants.ISCSI_VARIANTS
+        if storage_protocol in constants.FC_VARIANTS:
+            return constants.FC_VARIANTS
+        if storage_protocol in constants.NFS_VARIANTS:
+            return constants.NFS_VARIANTS
+        if storage_protocol in constants.NVMEOF_VARIANTS:
+            return constants.NVMEOF_VARIANTS
+        return storage_protocol
+
 
 class PoolState(BackendState):
     def __init__(self,
@@ -386,7 +409,7 @@ class PoolState(BackendState):
         self.pools: dict = {}
 
     def update_from_volume_capability(self,
-                                      capability: Dict[str, Any],
+                                      capability: dict[str, Any],
                                       service=None) -> None:
         """Update information about a pool from its volume_node info."""
         LOG.debug("Updating capabilities for %s: %s", self.host, capability)
@@ -449,7 +472,7 @@ class HostManager(object):
 
     def __init__(self):
         self.service_states = {}  # { <host|cluster>: {<service>: {cap k : v}}}
-        self.backend_state_map: Dict[str, BackendState] = {}
+        self.backend_state_map: dict[str, BackendState] = {}
         self.backup_service_states = {}
         self.filter_handler = filters.BackendFilterHandler('cinder.scheduler.'
                                                            'filters')
@@ -492,7 +515,7 @@ class HostManager(object):
 
     def _choose_backend_weighers(
             self,
-            weight_cls_names: Optional[List[str]]) -> list:
+            weight_cls_names: Optional[list[str]]) -> list:
         """Return a list of available weigher names.
 
         This function checks input weigher names against a predefined set
@@ -774,7 +797,7 @@ class HostManager(object):
 
     def get_pools(self,
                   context: cinder_context.RequestContext,
-                  filters: Optional[dict] = None) -> List[dict]:
+                  filters: Optional[dict] = None) -> list[dict]:
         """Returns a dict of all pools on all hosts HostManager knows about."""
 
         self._update_backend_state_map(context)
@@ -837,7 +860,7 @@ class HostManager(object):
                    capa_new: dict,
                    updated_pools: Iterable[dict],
                    host: str,
-                   timestamp) -> List[dict]:
+                   timestamp) -> list[dict]:
         pools = capa_new.get('pools')
         usage = []
         if pools and isinstance(pools, list):
@@ -867,7 +890,7 @@ class HostManager(object):
 
     def _get_pool_usage(self,
                         pool: dict,
-                        host: str, timestamp) -> Dict[str, Any]:
+                        host: str, timestamp) -> dict[str, Any]:
         total = pool["total_capacity_gb"]
         free = pool["free_capacity_gb"]
 
@@ -946,7 +969,7 @@ class HostManager(object):
 
     def _notify_capacity_usage(self,
                                context: cinder_context.RequestContext,
-                               usage: List[dict]) -> None:
+                               usage: list[dict]) -> None:
         if usage:
             for u in usage:
                 volume_utils.notify_about_capacity_usage(
@@ -967,12 +990,26 @@ class HostManager(object):
         # we just convert them into string to compare them.
         return str(value) == str(capability)
 
-    def get_backup_host(self, volume: objects.Volume, driver=None) -> str:
+    def get_az(self,
+               volume: objects.Volume,
+               availability_zone: Union[str, None]) -> Union[str, None]:
+        if availability_zone:
+            az = availability_zone
+        elif volume:
+            az = volume.availability_zone
+        else:
+            az = None
+        return az
+
+    def get_backup_host(self,
+                        volume: objects.Volume,
+                        availability_zone: Union[str, None],
+                        driver=None) -> str:
         if volume:
             volume_host = volume_utils.extract_host(volume.host, 'host')
         else:
             volume_host = None
-        az = volume.availability_zone if volume else None
+        az = self.get_az(volume, availability_zone)
         return self._get_available_backup_service_host(volume_host, az, driver)
 
     def _get_any_available_backup_service(self, availability_zone,
