@@ -157,6 +157,10 @@ class BackupCephTestCase(test.TestCase):
                 self.callstack.append('communicate')
                 return retval
 
+            def wait(mock_inst):
+                self.callstack.append('wait')
+                return retval
+
         subprocess.Popen.side_effect = MockPopen
 
     def setUp(self):
@@ -522,7 +526,8 @@ class BackupCephTestCase(test.TestCase):
                                               'popen_init',
                                               'write',
                                               'stdout_close',
-                                              'communicate'], self.callstack)
+                                              'communicate',
+                                              'wait'], self.callstack)
 
                             self.assertFalse(mock_full_backup.called)
                             self.assertFalse(mock_get_backup_snaps.called)
@@ -998,6 +1003,31 @@ class BackupCephTestCase(test.TestCase):
                 mock_getbasename.assert_called_once_with(self.volume_id)
                 mock_transfer_data.assert_called_once()
 
+    @common_mocks
+    def test_full_restore_with_image_not_found(self):
+        length = 1024
+        volume_is_new = True
+        src_snap = None
+        with tempfile.NamedTemporaryFile() as dest_file:
+            with mock.patch.object(self.service,
+                                   '_get_backup_base_name') as mock_name,\
+                    mock.patch('eventlet.tpool.Proxy') as mock_proxy:
+
+                self.mock_rbd.Image.side_effect = self.mock_rbd.ImageNotFound
+
+                self.assertRaises(self.mock_rbd.ImageNotFound,
+                                  self.service._full_restore,
+                                  self.backup,
+                                  dest_file,
+                                  length,
+                                  volume_is_new,
+                                  src_snap)
+
+                # Check that the _get_backup_base_name was called
+                # twice due to the exception
+                self.assertEqual(mock_name.call_count, 2)
+                self.assertEqual(mock_proxy.call_count, 2)
+
     @common_mocks
     def test_discard_bytes(self):
         # Lower the chunksize to a memory manageable number
@@ -1387,8 +1417,8 @@ class BackupCephTestCase(test.TestCase):
         mock_fcntl.return_value = 0
         self._setup_mock_popen(['out', 'err'])
         self.service._piped_execute(['foo'], ['bar'])
-        self.assertEqual(['popen_init', 'popen_init',
-                          'stdout_close', 'communicate'], self.callstack)
+        self.assertEqual(['popen_init', 'popen_init', 'stdout_close',
+                          'communicate', 'wait'], self.callstack)
 
     @common_mocks
     def test_restore_metdata(self):
