@@ -16,6 +16,7 @@
 """Unit tests for the Quobyte driver module."""
 
 import errno
+from io import StringIO
 import os
 import shutil
 import traceback
@@ -27,7 +28,6 @@ from oslo_utils import fileutils
 from oslo_utils import imageutils
 from oslo_utils import units
 import psutil
-import six
 
 from cinder import context
 from cinder import db
@@ -121,7 +121,7 @@ class QuobyteDriverTestCase(test.TestCase):
             self.assertIsInstance(exc, excClass,
                                   'Wrong exception caught: %s Stacktrace: %s' %
                                   (exc, traceback.format_exc()))
-            self.assertIn(msg, six.text_type(exc))
+            self.assertIn(msg, str(exc))
 
         if not caught:
             self.fail('Expected raised exception but nothing caught.')
@@ -273,7 +273,8 @@ class QuobyteDriverTestCase(test.TestCase):
 
         mock_qemu_img_info.assert_called_with(mock.sentinel.image_path,
                                               force_share=True,
-                                              run_as_root=True)
+                                              run_as_root=True,
+                                              allow_qcow2_backing_file=True)
 
     @ddt.data(['/other_random_path', '/mnt'],
               ['/other_basedir/' + TEST_MNT_HASH + '/volume-' + VOLUME_UUID,
@@ -375,7 +376,7 @@ class QuobyteDriverTestCase(test.TestCase):
                 mock.patch('cinder.volume.drivers.quobyte.QuobyteDriver'
                            '._validate_volume') as mock_validate:
             # Content of /proc/mount (not mounted yet).
-            mock_open.return_value = six.StringIO(
+            mock_open.return_value = StringIO(
                 "/dev/sda5 / ext4 rw,relatime,data=ordered 0 0")
 
             self._driver._mount_quobyte(self.TEST_QUOBYTE_VOLUME,
@@ -388,7 +389,7 @@ class QuobyteDriverTestCase(test.TestCase):
 
             mock_execute.assert_has_calls(
                 [mount_call], any_order=False)
-            mock_validate.called_once_with(self.TEST_MNT_POINT)
+            mock_validate.assert_called_once_with(self.TEST_MNT_POINT)
 
     def test_mount_quobyte_already_mounted_detected_seen_in_proc_mount(self):
         with mock.patch('cinder.volume.drivers.quobyte.QuobyteDriver'
@@ -396,7 +397,7 @@ class QuobyteDriverTestCase(test.TestCase):
                 mock.patch('cinder.volume.drivers.quobyte.QuobyteDriver'
                            '._validate_volume') as mock_validate:
             # Content of /proc/mount (already mounted).
-            mock_open.return_value = six.StringIO(
+            mock_open.return_value = StringIO(
                 "quobyte@%s %s fuse rw,nosuid,nodev,noatime,user_id=1000"
                 ",group_id=100,default_permissions,allow_other 0 0"
                 % (self.TEST_QUOBYTE_VOLUME, self.TEST_MNT_POINT))
@@ -423,7 +424,7 @@ class QuobyteDriverTestCase(test.TestCase):
                 mock.patch('cinder.volume.drivers.quobyte.QuobyteDriver'
                            '._validate_volume') as mock_validate:
             # Content of /proc/mount (empty).
-            mock_open.return_value = six.StringIO()
+            mock_open.return_value = StringIO()
             mock_execute.side_effect = [None, putils.ProcessExecutionError(
                 stderr='is busy or already mounted')]
 
@@ -452,7 +453,7 @@ class QuobyteDriverTestCase(test.TestCase):
                 mock.patch('oslo_utils.fileutils.ensure_tree') as mock_mkdir, \
                 mock.patch('cinder.volume.drivers.quobyte.QuobyteDriver'
                            '.read_proc_mount') as mock_open:
-            mock_open.return_value = six.StringIO()
+            mock_open.return_value = StringIO()
             mock_execute.side_effect = [
                 None,  # mkdir
                 putils.ProcessExecutionError(  # mount
@@ -941,13 +942,14 @@ class QuobyteDriverTestCase(test.TestCase):
                                                self.TEST_QUOBYTE_VOLUME),
                                            self.VOLUME_UUID)
 
-        qemu_img_info_output = """image: volume-%s
-        file format: qcow2
-        virtual size: 1.0G (1073741824 bytes)
-        disk size: 473K
-        """ % self.VOLUME_UUID
+        qemu_img_info_output = """{
+    "filename": "volume-%s",
+    "format": "qcow2",
+    "virtual-size": 1073741824,
+    "actual-size": 473000
+}""" % self.VOLUME_UUID
 
-        img_info = imageutils.QemuImgInfo(qemu_img_info_output)
+        img_info = imageutils.QemuImgInfo(qemu_img_info_output, format='json')
 
         self.mock_object(image_utils, 'qemu_img_info', return_value=img_info)
         self.mock_object(image_utils, 'resize_image')
@@ -960,10 +962,11 @@ class QuobyteDriverTestCase(test.TestCase):
         else:
             drv.extend_volume(volume, 3)
 
-            image_utils.qemu_img_info.assert_called_once_with(volume_path,
-                                                              force_share=True,
-                                                              run_as_root=False
-                                                              )
+            image_utils.qemu_img_info.assert_called_once_with(
+                volume_path,
+                force_share=True,
+                run_as_root=False,
+                allow_qcow2_backing_file=True)
             image_utils.resize_image.assert_called_once_with(volume_path, 3)
 
     def test_copy_volume_from_snapshot(self):
@@ -987,13 +990,14 @@ class QuobyteDriverTestCase(test.TestCase):
 
         size = dest_volume['size']
 
-        qemu_img_output = """image: %s
-        file format: raw
-        virtual size: 1.0G (1073741824 bytes)
-        disk size: 173K
-        backing file: %s
-        """ % (snap_file, src_volume['name'])
-        img_info = imageutils.QemuImgInfo(qemu_img_output)
+        qemu_img_output = """{
+    "filename": "%s",
+    "format": "raw",
+    "virtual-size": 1073741824,
+    "actual-size": 173000,
+    "backing-filename": "%s"
+}""" % (snap_file, src_volume['name'])
+        img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
 
         # mocking and testing starts here
         mock_convert = self.mock_object(image_utils, 'convert_image')
@@ -1006,9 +1010,11 @@ class QuobyteDriverTestCase(test.TestCase):
         drv._copy_volume_from_snapshot(snapshot, dest_volume, size)
 
         drv._read_info_file.assert_called_once_with(info_path)
-        image_utils.qemu_img_info.assert_called_once_with(snap_path,
-                                                          force_share=True,
-                                                          run_as_root=False)
+        image_utils.qemu_img_info.assert_called_once_with(
+            snap_path,
+            force_share=True,
+            run_as_root=False,
+            allow_qcow2_backing_file=True)
         (mock_convert.
          assert_called_once_with(src_vol_path,
                                  dest_vol_path,
@@ -1043,13 +1049,14 @@ class QuobyteDriverTestCase(test.TestCase):
 
         size = dest_volume['size']
 
-        qemu_img_output = """image: %s
-        file format: raw
-        virtual size: 1.0G (1073741824 bytes)
-        disk size: 173K
-        backing file: %s
-        """ % (snap_file, src_volume['name'])
-        img_info = imageutils.QemuImgInfo(qemu_img_output)
+        qemu_img_output = """{
+    "filename": "%s",
+    "format": "raw",
+    "virtual-size": 1073741824,
+    "actual-size": 173000,
+    "backing-filename": "%s"
+}""" % (snap_file, src_volume['name'])
+        img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
 
         # mocking and testing starts here
         mock_convert = self.mock_object(image_utils, 'convert_image')
@@ -1063,9 +1070,11 @@ class QuobyteDriverTestCase(test.TestCase):
         drv._copy_volume_from_snapshot(snapshot, dest_volume, size)
 
         drv._read_info_file.assert_called_once_with(info_path)
-        image_utils.qemu_img_info.assert_called_once_with(snap_path,
-                                                          force_share=True,
-                                                          run_as_root=False)
+        image_utils.qemu_img_info.assert_called_once_with(
+            snap_path,
+            force_share=True,
+            run_as_root=False,
+            allow_qcow2_backing_file=True)
         self.assertFalse(mock_convert.called,
                          ("_convert_image was called but should not have been")
                          )
@@ -1104,13 +1113,14 @@ class QuobyteDriverTestCase(test.TestCase):
 
         size = dest_volume['size']
 
-        qemu_img_output = """image: %s
-        file format: raw
-        virtual size: 1.0G (1073741824 bytes)
-        disk size: 173K
-        backing file: %s
-        """ % (snap_file, src_volume['name'])
-        img_info = imageutils.QemuImgInfo(qemu_img_output)
+        qemu_img_output = """{
+    "filename": "%s",
+    "format": "raw",
+    "virtual-size": 1073741824,
+    "actual-size": 173000,
+    "backing-filename": "%s"
+}""" % (snap_file, src_volume['name'])
+        img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
 
         # mocking and testing starts here
         mock_convert = self.mock_object(image_utils, 'convert_image')
@@ -1126,9 +1136,11 @@ class QuobyteDriverTestCase(test.TestCase):
         drv._read_info_file.assert_called_once_with(info_path)
         os_ac_mock.assert_called_once_with(
             drv._local_volume_from_snap_cache_path(snapshot), os.F_OK)
-        image_utils.qemu_img_info.assert_called_once_with(snap_path,
-                                                          force_share=True,
-                                                          run_as_root=False)
+        image_utils.qemu_img_info.assert_called_once_with(
+            snap_path,
+            force_share=True,
+            run_as_root=False,
+            allow_qcow2_backing_file=True)
         (mock_convert.
          assert_called_once_with(
              src_vol_path,
@@ -1168,13 +1180,14 @@ class QuobyteDriverTestCase(test.TestCase):
 
         size = dest_volume['size']
 
-        qemu_img_output = """image: %s
-        file format: raw
-        virtual size: 1.0G (1073741824 bytes)
-        disk size: 173K
-        backing file: %s
-        """ % (snap_file, src_volume['name'])
-        img_info = imageutils.QemuImgInfo(qemu_img_output)
+        qemu_img_output = """{
+    "filename": "%s",
+    "format": "raw",
+    "virtual-size": 1073741824,
+    "actual-size": 173000,
+    "backing-filename": "%s"
+}""" % (snap_file, src_volume['name'])
+        img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
 
         # mocking and testing starts here
         mock_convert = self.mock_object(image_utils, 'convert_image')
@@ -1188,9 +1201,11 @@ class QuobyteDriverTestCase(test.TestCase):
         drv._copy_volume_from_snapshot(snapshot, dest_volume, size)
 
         drv._read_info_file.assert_called_once_with(info_path)
-        image_utils.qemu_img_info.assert_called_once_with(snap_path,
-                                                          force_share=True,
-                                                          run_as_root=False)
+        image_utils.qemu_img_info.assert_called_once_with(
+            snap_path,
+            force_share=True,
+            run_as_root=False,
+            allow_qcow2_backing_file=True)
         (mock_convert.
          assert_called_once_with(
              src_vol_path,
@@ -1245,12 +1260,13 @@ class QuobyteDriverTestCase(test.TestCase):
                                drv._get_hash_str(self.TEST_QUOBYTE_VOLUME))
         vol_path = os.path.join(vol_dir, volume['name'])
 
-        qemu_img_output = """image: %s
-        file format: raw
-        virtual size: 1.0G (1073741824 bytes)
-        disk size: 173K
-        """ % volume['name']
-        img_info = imageutils.QemuImgInfo(qemu_img_output)
+        qemu_img_output = """{
+    "filename": "%s",
+    "format": "raw",
+    "virtual-size": 1073741824,
+    "actual-size": 173000
+}""" % volume['name']
+        img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
 
         drv.get_active_image_from_info = mock.Mock(return_value=volume['name'])
         self.mock_object(image_utils, 'qemu_img_info', return_value=img_info)
@@ -1258,9 +1274,11 @@ class QuobyteDriverTestCase(test.TestCase):
         conn_info = drv.initialize_connection(volume, None)
 
         drv.get_active_image_from_info.assert_called_once_with(volume)
-        image_utils.qemu_img_info.assert_called_once_with(vol_path,
-                                                          force_share=True,
-                                                          run_as_root=False)
+        image_utils.qemu_img_info.assert_called_once_with(
+            vol_path,
+            force_share=True,
+            run_as_root=False,
+            allow_qcow2_backing_file=True)
 
         self.assertEqual('raw', conn_info['data']['format'])
         self.assertEqual('quobyte', conn_info['driver_volume_type'])
@@ -1294,12 +1312,13 @@ class QuobyteDriverTestCase(test.TestCase):
 
             mock_create_temporary_file.return_value = self.TEST_TMP_FILE
 
-            qemu_img_output = """image: %s
-            file format: raw
-            virtual size: 1.0G (1073741824 bytes)
-            disk size: 173K
-            """ % volume['name']
-            img_info = imageutils.QemuImgInfo(qemu_img_output)
+            qemu_img_output = """{
+    "filename": "%s",
+    "format": "raw",
+    "virtual-size": 1073741824,
+    "actual-size": 173000
+}""" % volume['name']
+            img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
             mock_qemu_img_info.return_value = img_info
 
             upload_path = volume_path
@@ -1308,9 +1327,11 @@ class QuobyteDriverTestCase(test.TestCase):
 
             mock_get_active_image_from_info.assert_called_once_with(volume)
             mock_local_volume_dir.assert_called_once_with(volume)
-            mock_qemu_img_info.assert_called_once_with(volume_path,
-                                                       force_share=True,
-                                                       run_as_root=False)
+            mock_qemu_img_info.assert_called_once_with(
+                volume_path,
+                force_share=True,
+                run_as_root=False,
+                allow_qcow2_backing_file=True)
             mock_upload_volume.assert_called_once_with(
                 mock.ANY, mock.ANY, mock.ANY, upload_path, run_as_root=False,
                 store_id=None, base_image_ref=None, compress=True,
@@ -1346,12 +1367,13 @@ class QuobyteDriverTestCase(test.TestCase):
 
             mock_create_temporary_file.return_value = self.TEST_TMP_FILE
 
-            qemu_img_output = """image: %s
-            file format: qcow2
-            virtual size: 1.0G (1073741824 bytes)
-            disk size: 173K
-            """ % volume['name']
-            img_info = imageutils.QemuImgInfo(qemu_img_output)
+            qemu_img_output = """{
+    "filename": "%s",
+    "format": "qcow2",
+    "virtual-size": 1073741824,
+    "actual-size": 173000
+}""" % volume['name']
+            img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
             mock_qemu_img_info.return_value = img_info
 
             upload_path = self.TEST_TMP_FILE
@@ -1360,9 +1382,11 @@ class QuobyteDriverTestCase(test.TestCase):
 
             mock_get_active_image_from_info.assert_called_once_with(volume)
             mock_local_volume_dir.assert_called_with(volume)
-            mock_qemu_img_info.assert_called_once_with(volume_path,
-                                                       force_share=True,
-                                                       run_as_root=False)
+            mock_qemu_img_info.assert_called_once_with(
+                volume_path,
+                force_share=True,
+                run_as_root=False,
+                allow_qcow2_backing_file=True)
             mock_convert_image.assert_called_once_with(
                 volume_path, upload_path, 'raw', run_as_root=False)
             mock_upload_volume.assert_called_once_with(
@@ -1401,13 +1425,14 @@ class QuobyteDriverTestCase(test.TestCase):
 
             mock_create_temporary_file.return_value = self.TEST_TMP_FILE
 
-            qemu_img_output = """image: volume-%s.%s
-            file format: qcow2
-            virtual size: 1.0G (1073741824 bytes)
-            disk size: 173K
-            backing file: %s
-            """ % (self.VOLUME_UUID, self.SNAP_UUID, volume_filename)
-            img_info = imageutils.QemuImgInfo(qemu_img_output)
+            qemu_img_output = """{
+    "filename": "volume-%s.%s",
+    "format": "qcow2",
+    "virtual-size": 1073741824,
+    "actual-size": 173000,
+    "backing-filename": "%s"
+}""" % (self.VOLUME_UUID, self.SNAP_UUID, volume_filename)
+            img_info = imageutils.QemuImgInfo(qemu_img_output, format='json')
             mock_qemu_img_info.return_value = img_info
 
             upload_path = self.TEST_TMP_FILE
@@ -1416,9 +1441,11 @@ class QuobyteDriverTestCase(test.TestCase):
 
             mock_get_active_image_from_info.assert_called_once_with(volume)
             mock_local_volume_dir.assert_called_with(volume)
-            mock_qemu_img_info.assert_called_once_with(volume_path,
-                                                       force_share=True,
-                                                       run_as_root=False)
+            mock_qemu_img_info.assert_called_once_with(
+                volume_path,
+                force_share=True,
+                run_as_root=False,
+                allow_qcow2_backing_file=True)
             mock_convert_image.assert_called_once_with(
                 volume_path, upload_path, 'raw', run_as_root=False)
             mock_upload_volume.assert_called_once_with(
